@@ -208,6 +208,12 @@ typedef unsigned short etap_event_t;
 # include "h/systm.h"
 # include "h/time.h"
 
+# ifdef AFS_FBSD_ENV
+#  include <sys/socket.h>
+#  include <net/if.h>
+#  include <net/if_var.h>
+# endif
+
 # ifdef	AFS_AIX_ENV
 #  ifdef AFS_AIX41_ENV
 #   include "sys/statfs.h"
