@@ -2120,8 +2120,9 @@ afsd_run(void)
     }
     memset(inode_for_V, '\0', (cacheFiles * sizeof(AFSD_INO_T)));
     if (afsd_debug)
-	printf("%s: %d inode_for_V entries at 0x%x, %lu bytes\n", rn,
-	       cacheFiles, inode_for_V, (cacheFiles * sizeof(AFSD_INO_T)));
+	printf("%s: %d inode_for_V entries at %p, %lu bytes\n", rn,
+	       cacheFiles, inode_for_V,
+	       (unsigned long)cacheFiles * sizeof(AFSD_INO_T));
 #endif
 
     if (!(cacheFlags & AFSCALL_INIT_MEMCACHE)) {
